const app = getApp<IAppOption>()

Page({
  data: {
    popupShow: false,
    popupType: 'argument',
    popupParams: {
      position: 'bottom',
    },

    userInfo: {},
    bannerList: [],
    knowledgeList: [] as any,
    questionList: [] as any,

    pageTop: '',
    background: 'transparent',
  },
  onLoad() {
    app.waitLogin().then(() => {
      this.setData({})
      app.getUserInfo().then((userInfo) => {
        this.setData({
          userInfo,
          popupShow: !userInfo.guestPrivacyAgree,
        })
        if (!userInfo.guestPrivacyAgree) {
          app.mpBehavior({ PageName: 'PG_PATIENTLOGIN' })
        }
      })
      this.getBanner()
      this.getKnowledgeList()
      this.getQuestionList()
    })
  },
  getBanner() {
    wx.ajax({
      method: 'GET',
      url: '?r=wtx/common/banner-config',
      data: {},
    }).then((res) => {
      this.setData({
        bannerList: res,
      })
    })
  },
  getKnowledgeList() {
    wx.ajax({
      method: 'GET',
      url: '?r=wtx/knowledge/index-list',
      data: {},
    }).then((res) => {
      this.setData({
        knowledgeList: res.list,
      })
    })
  },
  getQuestionList() {
    wx.ajax({
      method: 'GET',
      url: '?r=wtx/common-question/index-list',
      data: {},
    }).then((res) => {
      this.setData({
        questionList: res.list,
      })
    })
  },
  onBannerItemTap(e: any) {
    app.mpBehavior({ PageName: 'BTN_HOMEBANNER' })
    const { path } = e.currentTarget.dataset
    wx.navigateTo({
      url: path,
    })
  },
  handleKnowledge() {
    app.mpBehavior({ PageName: 'BTN_HOMEEDUCATIONALL' })
    wx.switchTab({
      url: '/pages/knowledge/index',
    })
  },
  handleKnowledgeDetail(e: any) {
    app.mpBehavior({ PageName: 'BTN_HOMEEDUCATIONCARD' })
    const { index } = e.currentTarget.dataset
    const params = this.data.knowledgeList[index]
    if (params.Type == 1) {
      wx.navigateTo({
        url: `/pages/knowledgeDetail/index?id=${params.Id}`,
      })
      return
    }
    wx.navigateTo({
      url: `/pages/knowledgeDetailVideo/index?id=${params.Id}`,
    })
  },
  handleAskItem(e) {
    app.mpBehavior({ PageName: 'BTN_HOMEFAQENTRY' })

    const { id } = e.currentTarget.dataset
    wx.navigateTo({
      url: `/pages/ask/index?id=${id}`,
    })
    wx.ajax({
      method: 'POST',
      url: '?r=wtx/common-question/click',
      data: { Id: id },
    })
  },
  routerTo(e) {
    const { url } = e.currentTarget.dataset
    const PageName = {
      '/pages/applyFrom/index': 'BTN_HOMEPREAPPLYCARD',
      '/pages/ask/index': 'BTN_HOMEFAQENTRY',
    }[url]
    if (PageName) {
      app.mpBehavior({ PageName })
    }
    wx.navigateTo({
      url,
    })
  },
  handlePopupOk() {
    const { popupType } = this.data
    if (popupType === 'argument') {
      app.mpBehavior({ PageName: 'BTN_PATIENTLOG' })
      wx.ajax({
        method: 'POST',
        url: '?r=wtx/user/agree-guest-privacy',
        data: {
          WorkerId: app.globalData.scene?.workerId || '',
        },
      }).then(() => {
        this.setData({
          popupShow: false,
        })
      })
    }
  },
  handlePopupCancel() {
    if (this.data.popupType === 'argument') {
      wx.exitMiniProgram()
    }
  },
})

export {}
